/*
 * Copyright © 2013-2016 The Nxt Core Developers.
 * Copyright © 2016-2018 Jelurida IP B.V.
 *
 * See the LICENSE.txt file at the top-level directory of this distribution
 * for licensing information.
 *
 * Unless otherwise agreed in a custom licensing agreement with Jelurida B.V.,
 * no part of this software, including this file, may be copied, modified,
 * propagated, or distributed except according to the terms contained in the
 * LICENSE.txt file.
 *
 * Removal or modification of this copyright notice is prohibited.
 *
 */

package nxt.lightcontracts;

import nxt.Constants;
import nxt.NxtException;
import nxt.blockchain.Attachment;
import nxt.blockchain.ChainTransactionId;
import nxt.blockchain.TransactionType;
import nxt.util.Convert;
import org.json.simple.JSONObject;

import java.nio.ByteBuffer;

public final class ContractReferenceAttachment extends Attachment.AbstractAttachment {

    private final String contractName;
    private final String contractParams;
    private final ChainTransactionId contractId;

    ContractReferenceAttachment(ByteBuffer buffer) throws NxtException.NotValidException {
        super(buffer);
        this.contractName = Convert.readString(buffer, buffer.get(), Constants.MAX_CONTRACT_NAME_LENGTH).trim();
        this.contractParams = Convert.readString(buffer, buffer.get(), Constants.MAX_CONTRACT_PARAMS_LENGTH).trim();
        this.contractId = ChainTransactionId.parse(buffer);
    }

    ContractReferenceAttachment(JSONObject attachmentData) {
        super(attachmentData);
        this.contractName = Convert.nullToEmpty((String) attachmentData.get("contractName")).trim();
        this.contractParams = Convert.nullToEmpty((String) attachmentData.get("contractParams")).trim();
        this.contractId = ChainTransactionId.parse((JSONObject) attachmentData.get("contract"));
    }

    public ContractReferenceAttachment(String contractName, String contractParams, ChainTransactionId contractId) {
        this.contractName = contractName.trim();
        this.contractParams = Convert.nullToEmpty(contractParams).trim();
        this.contractId = contractId;
    }

    @Override
    protected int getMySize() {
        return 1 + Convert.toBytes(contractName).length + 1 + Convert.toBytes(contractParams).length + ChainTransactionId.BYTE_SIZE;
    }

    @Override
    protected void putMyBytes(ByteBuffer buffer) {
        byte[] name = Convert.toBytes(this.contractName);
        byte[] params = Convert.toBytes(this.contractParams);
        buffer.put((byte)name.length);
        buffer.put(name);
        buffer.put((byte)params.length);
        buffer.put(params);
        contractId.put(buffer);
    }

    @Override
    protected void putMyJSON(JSONObject attachment) {
        attachment.put("contractName", contractName);
        attachment.put("contractParams", contractParams);
        attachment.put("contract", contractId.getJSON());
    }

    @Override
    public TransactionType getTransactionType() {
        return LightContractTransactionType.CONTRACT_REFERENCE_SET;
    }

    public String getContractName() {
        return contractName;
    }

    public String getContractParams() {
        return contractParams;
    }

    public ChainTransactionId getContractId() {
        return contractId;
    }

}
